from .wrapper import Wrapper


class SqliteWrapper(Wrapper):
    def wrapper(self, obj):
        return sqlite_wrapper(obj)


def sqlite_wrapper(func):
    from django.db.backends.sqlite3.base import Database

    def null_converter(s):
        if isinstance(s, bytes):
            return s.decode("utf-8")
        return s

    def wrapper(*args, **kwargs):
        converter = Database.converters.pop("DATETIME")
        Database.register_converter("datetime", null_converter)
        res = func(*args, **kwargs)
        Database.register_converter("DATETIME", converter)
        return res

    return wrapper



from django.contrib.auth import get_user_model
from django.db import connections
from django.test import TestCase
from sqlalchemy import MetaData
from sqlalchemy.orm import aliased

from aldjemy.core import Cache, get_connection_string, get_engine
from aldjemy.orm import construct_models
from aldjemy.session import get_session
from test_project.sample.models import (
    Author,
    Book,
    BookProxy,
    Chapter,
    Item,
    Log,
    Person,
    RelatedToItemViaPrimaryKey,
    RelatedToItemViaUniqueField,
    Review,
    StaffAuthor,
    StaffAuthorProxy,
)

User = get_user_model()


class SimpleTest(TestCase):
    def test_aldjemy_initialization(self):
        self.assertTrue(Chapter.sa)
        self.assertTrue(Book.sa)
        self.assertTrue(Author.sa)
        self.assertTrue(StaffAuthor.sa)
        self.assertTrue(StaffAuthorProxy.sa)
        self.assertTrue(Review.sa)
        self.assertTrue(BookProxy.sa)
        self.assertTrue(User.sa)

        # Automatic Many to Many fields get the ``sa`` property
        books_field = Author._meta.get_field("books")
        self.assertTrue(books_field.remote_field.through.sa)

    def test_engine_override_test(self):
        self.assertEqual(get_connection_string(), "sqlite+pysqlite://")

    def test_querying(self):
        Book.objects.create(title="book title")
        Book.objects.all()
        self.assertEqual(Book.sa.query().count(), 1)

    def test_user_model(self):
        u = User.objects.create()
        Author.objects.create(user=u)
        a = Author.sa.query().first()
        self.assertEqual(a.user.id, u.id)


class AliasesTest(TestCase):
    databases = "__all__"

    def test_engines_cache(self):
        self.assertEqual(get_engine("default"), Cache.engines["default"])
        self.assertEqual(get_engine("logs"), Cache.engines["logs"])
        self.assertEqual(get_engine(), Cache.engines["default"])
        self.assertNotEqual(get_engine("default"), get_engine("logs"))

    def test_sessions(self):
        session_default = get_session()
        session_default2 = get_session("default")
        self.assertEqual(session_default, session_default2)
        session_logs = get_session("logs")
        self.assertEqual(connections["default"].sa_session, session_default)
        self.assertEqual(connections["logs"].sa_session, session_logs)
        self.assertNotEqual(session_default, session_logs)

    def test_logs(self):
        Log.objects.create(record="1")
        Log.objects.create(record="2")
        self.assertEqual(Log.objects.using("logs").count(), 2)
        self.assertEqual(Log.sa.query().count(), 2)
        self.assertEqual(Log.sa.query().all()[0].record, "1")


class AldjemyMetaTests(TestCase):
    databases = "__all__"

    def test_meta(self):
        Log.objects.create(record="foo")

        foo = Log.sa.query().one()
        self.assertEqual(str(foo), "foo")
        self.assertEqual(foo.reversed_record, "oof")
        self.assertFalse(hasattr(foo, "this_is_not_copied"))


class CustomMetaDataTests(TestCase):
    def test_custom_metadata_schema(self):
        """Use a custom MetaData instance to add a schema."""
        # The use-case for this functionality is to allow using
        # Foreign Data Wrappers, each with a full set of Django
        # tables, to copy between databases using SQLAlchemy
        # and the automatically generation of aldjemy.
        metadata = MetaData(schema="arbitrary")
        sa_models = construct_models(metadata)
        self.assertEqual(sa_models[Log].__table__.schema, "arbitrary")

    def test_custom_metadata_schema_aliased(self):
        """Make sure the aliased command works with the schema."""
        # This was an issue that cropped up after things seemed
        # to be generating properly, so we want to test it and
        # make sure that it stays working.
        metadata = MetaData(schema="pseudorandom")
        sa_models = construct_models(metadata)
        aliased(sa_models[Log])

    def test_many_to_many_through_self(self):
        """Make sure recursive through tables work."""
        through_field = Person._meta.get_field("parents")
        through = through_field.remote_field.through

        metadata = MetaData(schema="unique")
        sa_models = construct_models(metadata)
        self.assertEqual(sa_models[through].__table__.schema, "unique")

    def test_many_to_many_through_self_aliased(self):
        """Make sure aliased recursive through tables work."""
        through_field = Person._meta.get_field("parents")
        through = through_field.remote_field.through

        metadata = MetaData(schema="unique")
        sa_models = construct_models(metadata)
        aliased(sa_models[through])


class ForeignKeyTests(TestCase):
    def test_foreign_key_through_pk(self):
        """Make sure that foreign keys to primary keys work."""
        metadata = MetaData(schema="unique")
        sa_models = construct_models(metadata)
        sa_model = sa_models[RelatedToItemViaPrimaryKey]
        table = sa_model.__table__
        self.assertEqual(len(table.foreign_keys), 1)
        foreign_key, *_ = table.foreign_keys
        foreign_column = foreign_key.column
        item_table = sa_models[Item].__table__
        self.assertIs(foreign_column.table, item_table)
        self.assertEqual(foreign_column.name, "id")
        self.assertEqual(foreign_column.type, item_table.c.id.type)

    def test_foreign_key_through_unique_field(self):
        """Make sure that foreign keys to unique but non primary columns work."""
        metadata = MetaData(schema="unique")
        sa_models = construct_models(metadata)
        sa_model = sa_models[RelatedToItemViaUniqueField]
        table = sa_model.__table__
        self.assertEqual(len(table.foreign_keys), 1)
        foreign_key, *_ = table.foreign_keys
        foreign_column = foreign_key.column
        item_table = sa_models[Item].__table__
        self.assertIs(foreign_column.table, item_table)
        self.assertEqual(foreign_column.name, "legacy_id")
        self.assertEqual(foreign_column.type, item_table.c.legacy_id.type)



class LogsRouter:
    ALIAS = "logs"

    def use_logs(self, model):
        return hasattr(model, "_DATABASE") and model._DATABASE == self.ALIAS

    def db_for_read(self, model, **hints):
        if self.use_logs(model):
            return self.ALIAS

    def db_for_write(self, model, **hints):
        return self.db_for_read(model, **hints)

    def allow_migrate(self, db, app_label, model_name=None, **hints):
        model = hints.get("model", None)
        if db == "logs":
            return self.use_logs(model)



# Generated by Django 3.2.3 on 2021-05-21 09:31

import django.contrib.postgres.fields.ranges
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("pg", "0003_jsonmodel"),
    ]

    operations = [
        migrations.CreateModel(
            name="DateRangeModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("date_range", django.contrib.postgres.fields.ranges.DateRangeField()),
            ],
        ),
    ]



from aldjemy.table import foreign_key

DEFAULT_AUTO_FIELD = "django.db.models.AutoField"
SECRET_KEY = "not-a-secret"
ROOT_URLCONF = "test_project.urls"
SITE_ID = 1
USE_TZ = False  # Silence a warning until Django 5.0

ALDJEMY_ENGINES = {"sqlite3": "sqlite+pysqlite"}
DATABASE_ROUTERS = [
    "test_project.pg.routers.PgRouter",
    "test_project.sample.routers.LogsRouter",
]

ALDJEMY_DATA_TYPES = {
    "AFakeType": foreign_key,
}

INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "test_project.sample",
    "test_project.pg",
    "aldjemy",
]

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": "default.db",
    },
    "logs": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": "logs.db",
    },
    "pg": {
        "ENGINE": "django.db.backends.postgresql_psycopg2",
        "NAME": "aldjemy",
        "USER": "postgres",
        "PASSWORD": "postgres",
        "HOST": "localhost",
        "PORT": "5432",
    },
}



from django.contrib.postgres.fields import ArrayField, DateRangeField
from django.db import models


class TicTacToeBoard(models.Model):
    board = ArrayField(models.TextField(), size=9)


class JsonModel(models.Model):
    value = models.JSONField()


class DateRangeModel(models.Model):
    date_range = DateRangeField()


class DecimalArrayModel(models.Model):
    array = ArrayField(
        models.DecimalField(decimal_places=3, max_digits=5), max_length=10
    )



from typing import Callable

from django.apps import apps
from django.db.models.fields.related import ForeignKey, ManyToManyField, OneToOneField
from sqlalchemy import orm
from sqlalchemy.orm import registry

from .table import generate_tables


def _extract_model_attrs(metadata, model, sa_models):
    tables = metadata.tables

    name = model._meta.db_table
    qualname = (metadata.schema + "." + name) if metadata.schema else name
    table = tables[qualname]
    fks = [t for t in model._meta.fields if isinstance(t, (ForeignKey, OneToOneField))]
    attrs = {}
    rel_fields = fks + list(model._meta.many_to_many)

    for f in model._meta.fields:
        if not isinstance(f, (ForeignKey, OneToOneField)):
            if f.model != model or f.column not in table.c:
                continue  # Fields from parent model are not supported
            attrs[f.name] = orm.column_property(table.c[f.column])

    for fk in rel_fields:
        if fk.column not in table.c and not isinstance(fk, ManyToManyField):
            continue

        parent_model = fk.remote_field.model

        parent_model_meta = parent_model._meta

        if parent_model_meta.proxy:
            continue

        p_table_name = parent_model_meta.db_table
        p_table_qualname = (
            metadata.schema + "." + p_table_name if metadata.schema else p_table_name
        )
        p_table = tables[p_table_qualname]
        p_name = parent_model_meta.pk.column

        related_name = fk.remote_field.get_accessor_name()
        disable_backref = related_name and related_name.endswith("+")
        backref = related_name.lower().strip("+") if related_name else None
        if not backref and not disable_backref:
            backref = model._meta.object_name.lower()
            if not isinstance(fk, OneToOneField):
                backref = backref + "_set"
        elif backref and isinstance(fk, OneToOneField):
            backref = orm.backref(backref, uselist=False)

        kwargs = {}
        if isinstance(fk, ManyToManyField):
            model_pk = model._meta.pk.column
            sec_table_name = fk.remote_field.field.m2m_db_table()
            sec_table_qualname = (
                metadata.schema + "." + sec_table_name
                if metadata.schema
                else sec_table_name
            )
            sec_table = tables[sec_table_qualname]
            sec_column = fk.m2m_column_name()
            p_sec_column = fk.m2m_reverse_name()
            overlaps = [
                fk.m2m_field_name(),
                fk.m2m_reverse_field_name(),
                fk.remote_field.field.get_attname(),
                fk.remote_field.through._meta.get_field(
                    fk.remote_field.field.m2m_field_name()
                ).remote_field.get_accessor_name(),
                fk.remote_field.through._meta.get_field(
                    fk.m2m_reverse_field_name()
                ).remote_field.get_accessor_name(),
            ]
            kwargs.update(
                secondary=sec_table,
                primaryjoin=(sec_table.c[sec_column] == table.c[model_pk]),
                secondaryjoin=(sec_table.c[p_sec_column] == p_table.c[p_name]),
                overlaps=",".join(overlaps),
            )
            if fk.model() != model:
                backref = None
        else:
            kwargs.update(
                foreign_keys=[table.c[fk.column]],
                primaryjoin=(table.c[fk.column] == p_table.c[p_name]),
                remote_side=p_table.c[p_name],
            )
            if backref and not disable_backref:
                kwargs.update(backref=backref)
        attrs[fk.name] = orm.relationship(sa_models[parent_model], **kwargs)
    return attrs


def _default_make_sa_model(model):
    """Create a custom class for the SQLAlchemy model."""
    name = model._meta.object_name + ".__aldjemy__"
    return type(name, (), {"__module__": model.__module__})


def construct_models(metadata, *, _make_sa_model: Callable = _default_make_sa_model):
    if not metadata.tables:
        generate_tables(metadata)
    tables = metadata.tables
    models = [
        model
        for model in apps.get_models(include_auto_created=True)
        if not model._meta.proxy
    ]

    sa_models = {}
    for model in models:
        table_name = (
            metadata.schema + "." + model._meta.db_table
            if metadata.schema
            else model._meta.db_table
        )
        table = tables[table_name]
        sa_models[model] = _make_sa_model(model)

    mapper_registry = registry()
    for model in models:
        sa_model = sa_models[model]
        table_name = (
            metadata.schema + "." + model._meta.db_table
            if metadata.schema
            else model._meta.db_table
        )
        table = tables[table_name]
        attrs = _extract_model_attrs(metadata, model, sa_models)
        mapper_registry.map_imperatively(sa_model, local_table=table, properties=attrs)

    return sa_models



from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _

from aldjemy.meta import AldjemyMeta


class Group(models.Model):
    """A model with the same name as a Django model to trigger some warnings."""

    # https://github.com/aldjemy/aldjemy/issues/205


class Chapter(models.Model):
    title = models.CharField(max_length=200)
    book = models.ForeignKey("Book", on_delete=models.CASCADE)


class Book(models.Model):
    title = models.CharField(max_length=200)


class ArticleAuthorAssociation(models.Model):
    author = models.ForeignKey(
        "Author", on_delete=models.CASCADE, related_name="article_associations"
    )
    article = models.ForeignKey(
        "Article", on_delete=models.CASCADE, related_name="author_associations"
    )


class Article(models.Model):
    title = models.CharField(max_length=200)


class BookProxy(Book):
    class Meta:
        proxy = True


class Author(models.Model):
    name = models.CharField(max_length=200)
    biography = models.TextField()
    books = models.ManyToManyField(Book, related_name="books")
    article = models.ManyToManyField(
        Article, through=ArticleAuthorAssociation, related_name="authors"
    )
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)


class StaffAuthor(Author):
    role = models.TextField()
    date = models.DateTimeField()


class StaffAuthorProxy(Author):
    class Meta:
        proxy = True


class Review(models.Model):
    book = models.ForeignKey("BookProxy", on_delete=models.CASCADE)


class Log(models.Model, metaclass=AldjemyMeta):
    _DATABASE = "logs"

    record = models.CharField(max_length=100)

    def __str__(self):
        return self.record

    @property
    def reversed_record(self):
        return self.record[::-1]

    this_is_not_copied = "something"


class Person(models.Model):
    parents = models.ManyToManyField("self", related_name="children")


class MemberCollective(models.Model):
    name = models.CharField(_("name"), max_length=20, null=False, blank=False)


class Membership(models.Model):
    member = models.ForeignKey("Member", on_delete=models.CASCADE)
    member_collective = models.ForeignKey(
        "MemberCollective", on_delete=models.CASCADE, related_name="member_memberships"
    )


class Member(models.Model):
    name = models.CharField(_("name"), max_length=255, null=False, blank=False)

    collectives = models.ManyToManyField(
        MemberCollective,
        related_name="members",
        through="Membership",
    )


class Item(models.Model):
    label = models.TextField(null=False, blank=False)
    legacy_id = models.CharField(max_length=255, unique=True, null=False, blank=False)


class RelatedToItemViaPrimaryKey(models.Model):
    item = models.ForeignKey(Item, on_delete=models.CASCADE)


class RelatedToItemViaUniqueField(models.Model):
    item = models.ForeignKey(Item, to_field="legacy_id", on_delete=models.CASCADE)



from django.conf import settings
from django.db import connections
from sqlalchemy import orm

from .core import get_engine

SQLALCHEMY_USE_FUTURE = getattr(settings, "ALDJEMY_SQLALCHEMY_USE_FUTURE", None)


def get_session(alias="default", recreate=False, **kwargs):
    connection = connections[alias]
    if not hasattr(connection, "sa_session") or recreate:
        engine = get_engine(alias, **kwargs)
        kwargs = {"bind": engine}
        if SQLALCHEMY_USE_FUTURE is not None:
            kwargs["future"] = SQLALCHEMY_USE_FUTURE  # pragma: no cover
        session = orm.sessionmaker(**kwargs)
        connection.sa_session = session()
    return connection.sa_session



class PgRouter:
    def db_for_read(self, model, **hints):
        if model._meta.app_label == "pg":
            return "pg"

    def db_for_write(self, model, **hints):
        return self.db_for_read(model, **hints)

    def allow_migrate(self, db, app_label, model_name=None, **hints):
        if app_label == "pg" or db == "pg":
            return app_label == "pg" and db == "pg"



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2017-07-05 13:16
from __future__ import unicode_literals

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="TicTacToeBoard",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "board",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.CharField(blank=True, max_length=1), size=9
                    ),
                ),
            ],
        ),
    ]



from django.db import transaction
from django.test import TestCase, TransactionTestCase
from sqlalchemy import select
from sqlalchemy.dialects.postgresql import array

from aldjemy.core import DjangoPool, get_engine
from aldjemy.session import get_session
from test_project.pg.models import (
    DateRangeModel,
    DecimalArrayModel,
    JsonModel,
    TicTacToeBoard,
)


class TestArrayField(TransactionTestCase):
    """
    Tests that queries involving array fields can be performed.
    """

    databases = ["pg"]

    def test_tic_tac_toe(self):
        """
        Test querying the TicTacToeBoard model.
        """
        boards = [
            ["x", "o", "x", "o", "o", "x", "x", "x", "o"],  # both (full board)
            [" ", " ", " ", " ", "x", " ", " ", " ", " "],  # only x
            [" ", " ", " ", "o", "o", " ", " ", " ", "o"],  # only o
            [" ", " ", " ", " ", " ", " ", " ", " ", " "],  # none
        ]
        for board in boards:
            ttt = TicTacToeBoard(board=board)
            ttt.save()

        contains = lambda c: TicTacToeBoard.sa.board.contains(array([c]))

        query = TicTacToeBoard.sa.query(TicTacToeBoard.sa.id)

        assert query.filter(contains("x")).count() == 2
        assert query.filter(contains("o")).count() == 2
        assert query.filter(contains(" ")).count() == 3

    def test_sa_objects_fetching(self):
        """
        Test full object fetching using SQLAlchemy-aldjemy ORM.
        """
        boards = [
            ["x", "o", "x", "o", "o", "x", "x", "x", "o"],  # both (full board)
            [" ", " ", " ", " ", "x", " ", " ", " ", " "],  # only x
            [" ", " ", " ", "o", "o", " ", " ", " ", "o"],  # only o
            [" ", " ", " ", " ", " ", " ", " ", " ", " "],  # none
        ]

        created_objects = []
        for board in boards:
            ttt = TicTacToeBoard(board=board)
            ttt.save()
            created_objects.append(ttt)

        session = get_session("pg")
        test_object = session.get(TicTacToeBoard.sa, created_objects[0].id)
        assert test_object.id == created_objects[0].id
        assert test_object.board == boards[0]

    def test_sa_sql_expression_language_fetching(self):
        """
        Test full record fetching using SQLAlchemy-aldjemy SQL Expression Language.
        """
        boards = [
            ["x", "o", "x", "o", "o", "x", "x", "x", "o"],  # both (full board)
            [" ", " ", " ", " ", "x", " ", " ", " ", " "],  # only x
            [" ", " ", " ", "o", "o", " ", " ", " ", "o"],  # only o
            [" ", " ", " ", " ", " ", " ", " ", " ", " "],  # none
        ]

        created_objects = []
        for board in boards:
            ttt = TicTacToeBoard(board=board)
            ttt.save()
            created_objects.append(ttt)

        query = (
            select(TicTacToeBoard.sa.id, TicTacToeBoard.sa.board)
            .order_by(TicTacToeBoard.sa.id)
            .limit(10)
        )

        with get_engine("pg").begin() as connection:
            test_data = connection.execute(query)

        for t_data, c_object in zip(test_data, created_objects):
            t_data_id, t_data_board = t_data
            assert t_data_id == c_object.id
            assert t_data_board == c_object.board


class TestJsonField(TestCase):
    def test_model_creates(self):
        """
        It's important that the field not cause the project to fail startup.
        """
        assert JsonModel.sa is not None


class TestDateRangeField(TestCase):
    def test_model_creates(self):
        assert DateRangeModel.sa is not None


class TestDecimalArrayField(TestCase):
    def test_model_creates(self):
        assert DecimalArrayModel.sa is not None


class RegressionTests(TestCase):
    databases = ["pg"]

    def test_transaction_is_not_rolled_back(self):
        """
        https://github.com/aldjemy/aldjemy/issues/173
        We are using savepoints in the hope of reproducing the conditions
        that helped us identify a regression introduced by SQLAlchemy 1.4.

        A ROLLBACK is emitted on every re-usage of a connection.
        This is undesirable, since we assume django owns the connection
        and the transaction state that comes with it.
        """
        board = [" "] * 9
        with transaction.atomic("pg"):
            tic_tac_toe = TicTacToeBoard.objects.create(board=board)
            # Just initiate a connection with SQLA, because the transation is rollback
            # the data we just created with django ORM is not available
            # Hence sqlalchemy.exc.NoResultFound is raised

            # if the exception is not raised, we assume, no ROLLBACK is emitted,
            # Indicating we successfully disabled the ROLLBACK.
            TicTacToeBoard.sa.query().filter(
                TicTacToeBoard.sa.id == tic_tac_toe.id
            ).one()


class DjangoPoolTest(TestCase):
    def test_pool_can_recreate(self):
        """DjangoPool can be created and recreated without errors."""
        pool = DjangoPool("test_alias", creator=None)
        pool.recreate()



from django.apps import AppConfig
from django.conf import settings
from django.db import router
from django.db.backends import signals
from sqlalchemy import MetaData

from .orm import construct_models
from .session import get_session


def new_session(sender, connection, **kwargs):
    """Create a new session for the given connection."""
    if connection.alias in settings.DATABASES:
        get_session(alias=connection.alias, recreate=True)


class BaseSQLAModel:
    @classmethod
    def query(cls, *args, **kwargs):
        alias = getattr(cls, "__alias__", "default")
        if args or kwargs:
            return get_session(alias).query(*args, **kwargs)
        return get_session(alias).query(cls)


def _make_sa_model(model):
    """Create a custom class for the SQLAlchemy model."""
    mixin = getattr(model, "aldjemy_mixin", None)
    bases = (mixin, BaseSQLAModel) if mixin else (BaseSQLAModel,)

    # because querying happens on sqlalchemy side, we can use only one
    # type of queries for alias, so we use 'read' type
    return type(
        model._meta.object_name + ".sa",
        bases,
        {
            "__alias__": router.db_for_read(model),
            "__module__": model.__module__,
        },
    )


class AldjemyConfig(AppConfig):
    name = "aldjemy"
    verbose_name = "Aldjemy"

    def ready(self):
        # Patch models with SQLAlchemy models
        models = construct_models(MetaData(), _make_sa_model=_make_sa_model)
        for model, sa_model in models.items():
            model.sa = sa_model

        signals.connection_created.connect(new_session)



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2017-07-05 13:31
from __future__ import unicode_literals

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("pg", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="tictactoeboard",
            name="board",
            field=django.contrib.postgres.fields.ArrayField(
                base_field=models.TextField(), size=9
            ),
        ),
    ]



# Generated by Django 5.0.4 on 2024-04-12 03:24

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("pg", "0004_daterangemodel"),
    ]

    operations = [
        migrations.CreateModel(
            name="DecimalArrayModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "array",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.DecimalField(decimal_places=3, max_digits=5),
                        max_length=10,
                        size=None,
                    ),
                ),
            ],
        ),
    ]
